<?php

namespace FM\Factory;

use Doctrine\ORM\EntityManager;
use FM\Traits\TQuery;

class AppOfflineFactory
{

    use TQuery;

    protected $connection;
    protected $entityManager;

    public function __construct(EntityManager $entityManager)
    {
        $this->connection       = $entityManager->getConnection();
        $this->entityManager    = $entityManager;
    }

    public function getDadosTarefaOffline($listaOperadores, $filtros)
    {
        $sql = "SELECT CA.NRCHECKLISTAGEND, CA.NRLOCALCLIENTE, CA.NRCHECKLIST, CA.CDOPERADOR, CA.IDSITUACAOAGEND, CA.NRSEQCONTRATO, LC.CDCLIENTE,
                    TO_CHAR(CA.DTHORAAGEND, 'DD/MM') DTAGEND,  TO_CHAR(CA.DTHORAAGEND, 'HH24:MI') HORAAGEND, CA.NMCHECKLISTAGEND, LC.NMLOCALCLIENTE, LC.NRLOCALCLIENTEPAI AS LOCALPAI,
                    LCP.NMLOCALCLIENTE AS NMLOCALPAI, CHK.NMCHECKLIST, F.NMOPERADOR, C.NRCONTRATOFCT, CT.NMRAZSOCCLIE, CA.IDOBRIGAOBSERVACAO, CA.IDTIPOMARCACAOTAREFA, DTHORAAGEND, S.NRSOLICITACAO,
                                    CASE CA.IDSITUACAOAGEND 
                                    WHEN '1' THEN 'Pendente'
                                    WHEN '2' THEN 'Em andamento'
                                    WHEN '3' THEN 'Cancelado'
                                    WHEN '4' THEN 'Concluído'
                                    WHEN '5' THEN 'Atrasado'
                                    ELSE NULL END AS SITUACAO
                                FROM FCM_CHECKLISTAGEND CA
                                JOIN FCM_LOCALCLI LC
                                ON LC.NRLOCALCLIENTE = CA.NRLOCALCLIENTE
                                JOIN FCM_LOCALCLI LCP
                                ON LCP.NRLOCALCLIENTE = LC.NRLOCALCLIENTEPAI
                                LEFT JOIN CLIENTE CT
                                ON CT.CDCLIENTE = LC.CDCLIENTE
                                LEFT JOIN FCM_CHECKLIST CHK
                                ON CA.NRCHECKLIST = CHK.NRCHECKLIST
                                JOIN OPERADOR F
                                ON F.CDOPERADOR = CA.CDOPERADOR
                                LEFT JOIN FCT_CONTRATO C
                                ON C.NRSEQCONTRATO = CA.NRSEQCONTRATO
                                LEFT JOIN FCM_SOLICITACAOTAREFAS S
                                ON S.NRCHECKLISTAGEND = CA.NRCHECKLISTAGEND
                                WHERE CA.IDSITUACAOAGEND IN (1,2,5)
                                AND CA.IDATIVO = 'S'
                                AND CA.CDOPERADOR IN ({$listaOperadores})" . $filtros . "
                                ORDER BY CA.IDSITUACAOAGEND DESC, CA.DTHORAAGEND ASC";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function getEpisTarefaOffline()
    {
        $sql = "SELECT FCM_RELCHKAGENDEPI.NRRELCHKAGENDEPI, FCM_RELCHKAGENDEPI.NREPI, FCM_EPI.NMEPI
                    FROM FCM_RELCHKAGENDEPI
                    JOIN FCM_EPI
                    ON FCM_EPI.NREPI = FCM_RELCHKAGENDEPI.NREPI";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function getFotosTarefasOffline()
    {
        $sql = "SELECT FCM_FOTOTAREFAAGEND.NRFOTOTAREFAAGEND, FCM_FOTOTAREFAAGEND.DSFOTOTAREFAAGEND, FCM_CHECKLISTAGEND.NRCHECKLISTAGEND, FCM_CHECKLISTAGEND.NMCHECKLISTAGEND
                    FROM FCM_FOTOTAREFAAGEND
                    JOIN FCM_TAREFAAGEND 
                    ON FCM_TAREFAAGEND.NRTAREFAAGEND = FCM_FOTOTAREFAAGEND.NRTAREFAAGEND
                    JOIN FCM_CHECKLISTAGEND 
                    ON FCM_CHECKLISTAGEND.NRCHECKLISTAGEND = FCM_TAREFAAGEND.NRCHECKLISTAGEND";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function getItensTarefasOffline($listaOperadores)
    {
        $sql = "SELECT TA.NRTAREFAAGEND, TA.NRCHECKLISTAGEND, TA.CDOPERADOR, TO_CHAR(TA.DTHORATAREFAAGEND, 'DD/MM HH24:MI') DTHORATAREFAAGEND,
              TA.NRTAREFACHK, TA.IDREALIZADO, TF.NMTAREFA
               FROM FCM_TAREFAAGEND TA
               JOIN FCM_CHECKLISTAGEND  CA
               ON TA.NRCHECKLISTAGEND = CA.NRCHECKLISTAGEND
               JOIN FCM_TAREFACHK T
               ON T.NRTAREFACHK = TA.NRTAREFACHK
               JOIN FCM_TAREFA TF
               ON T.NRTAREFA = TF.NRTAREFA
               JOIN OPERADOR F
               ON F.CDOPERADOR = CA.CDOPERADOR
               WHERE CA.CDOPERADOR IN ({$listaOperadores})"; 
        
        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function getParamOffline()
    {
        $sql = "SELECT IDHABILITAOFFLINE, IDTIPOCARREGAOFFLINE, QTDIASCARREGAOFFLINE, TO_CHAR(DTINICARREGAOFFLINE, 'DD/MM/YYYY') DTINICAL, 
                       TO_CHAR(DTFIMCARREGAOFFLINE, 'DD/MM/YYYY') DTFINAL
                FROM FCM_PARAMGERAL";

        return $this->connection->executeQuery($sql)->fetchAllAssociative();
    }

    public function getNrParametrizacoes()
    {
        $sql = "SELECT NRPARAMGERAL
                FROM FCM_PARAMGERAL";

        return $this->connection->executeQuery($sql)->fetchAllAssociative();
    }

    public function getOrdenacoes($param)
    {
        $sql = "SELECT NRORDENCAOAPP, NMTELA, NMCAMPO, IDSENTIDO
                    FROM FCM_ORDENACAOAPP
                    WHERE CDOPERADOR = :CDOPERADOR";
        
        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $param);
    }

    public function atualizarParametrizacao($param)
    {
        $sql = "UPDATE FCM_PARAMGERAL
                   SET IDHABILITAOFFLINE     = :IDHABILITAOFFLINE,
                       DTINICARREGAOFFLINE   = :DTINICARREGAOFFLINE,
                       DTFIMCARREGAOFFLINE   = :DTFIMCARREGAOFFLINE,
                       CDOPERULTATU          = :CDOPERULTATU,
                       NRORGULTATU           = :NRORGULTATU,
                       DTULTATU              = SYSDATE,
                       QTDIASCARREGAOFFLINE  = :QTDIASCARREGAOFFLINE,
                       IDTIPOCARREGAOFFLINE  = :IDTIPOCARREGAOFFLINE
                 WHERE NRPARAMGERAL          = :NRPARAMGERAL";

        $this->connection->executeStatement($sql, $param);
    }

    public function getLocalCliente()
    {
        $sql = "SELECT NRLOCALCLIENTE, NMLOCALCLIENTE, CDCLIENTE, NRTIPOLOCALCLI, NRLOCALCLIENTEPAI, NRCHECKLISTPADR, NRGEOLOCALIZACAO
                FROM FCM_LOCALCLI";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function getParametrizacaoGeral()
    {
        $sql = "SELECT NRPARAMGERAL, IDOBRIGACDLOCAT, IDUTILIZANOTAAUDIT, IDUTILIZAPERCENTUALAUDIT, IDHABILITAOFFLINE, IDUTILIZAGEOLOCAL, IDUTILIZAGEOLOCALINITAR
                FROM FCM_PARAMGERAL";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function getModeloPopOffline()
    {
        $sql = "SELECT FCM_RELCHKAGENDMODPOP.NRRELCHKAGENDMODPOP, FCM_RELCHKAGENDMODPOP.NRMODELOPOP, FCM_MODELOPOP.NMMODELOPOP,
                    FCM_RELCHKAGENDMODPOP.DSOBSERVACAO, FCM_MODELOPOP.DESCMODELOPOP, FCM_MODELOPOP.LINKMODELOPOP, FCM_ANEXOMODELOPOP.DSCAMINHOARQ, FCM_ANEXOMODELOPOP.NRANEXOMODELOPOP
                    FROM FCM_RELCHKAGENDMODPOP
                    JOIN FCM_MODELOPOP
                    ON FCM_MODELOPOP.NRMODELOPOP =FCM_RELCHKAGENDMODPOP.NRMODELOPOP
                    JOIN FCM_ANEXOMODELOPOP
                    ON FCM_ANEXOMODELOPOP.NRMODELOPOP =FCM_RELCHKAGENDMODPOP.NRMODELOPOP";
        
        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function getCriteriosOffline()
    {

        $sql = "SELECT FCM_CRITERIOTARGERCHK.NRCRITERIOTARGERCHK, FCM_CRITERIOTARGERCHK.NMCRITERIOTARGERCHK, FCM_CRITERIOTARGERCHK.IDMARCADO, FCM_TAREFACHK.NRATIVIDADECHK, FCM_TAREFAGERCHK.NRGERENCIAMENTOCHK
                    FROM FCM_CRITERIOTARGERCHK
                    JOIN FCM_TAREFAGERCHK
                    ON FCM_CRITERIOTARGERCHK.NRTAREFAGERCHK = FCM_TAREFAGERCHK.NRTAREFAGERCHK
                    JOIN FCM_TAREFACHK
                    ON FCM_TAREFAGERCHK.NRTAREFACHK = FCM_TAREFACHK.NRTAREFACHK
                    JOIN FCM_TAREFA 
                    ON FCM_TAREFA.NRTAREFA = FCM_TAREFACHK.NRTAREFA";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function getAtividadesOffline()
    {

        $sql = "SELECT DISTINCT 
            FCM_ATIVIDADECHK.NRATIVIDADECHK, 
            FCM_ATIVIDADE.NMATIVIDADE, 
            FCM_TAREFAGERCHK.NRGERENCIAMENTOCHK
        FROM FCM_ATIVIDADECHK
        JOIN FCM_ATIVIDADE
            ON FCM_ATIVIDADE.NRATIVIDADE = FCM_ATIVIDADECHK.NRATIVIDADE
        JOIN FCM_TAREFACHK
            ON FCM_TAREFACHK.NRATIVIDADECHK = FCM_ATIVIDADECHK.NRATIVIDADECHK
        JOIN FCM_TAREFAGERCHK
            ON FCM_TAREFAGERCHK.NRTAREFACHK = FCM_TAREFACHK.NRTAREFACHK";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function getCategoriasAuditoriaOffline()
    {
        $sql = "SELECT 
                    ROWNUM AS id,
                    FCM_RELACMODCATEG.*, 
                    FCM_CATEGORIAAUDIT.NMCATEGORIAAUDIT, 
                    FCM_AUDITORIA.NRAUDITORIA
                FROM FCM_RELACMODCATEG
                JOIN FCM_AUDITORIA
                    ON FCM_AUDITORIA.NRMODELOAUDIT = FCM_RELACMODCATEG.NRMODELOAUDIT
                JOIN FCM_CATEGORIAAUDIT
                    ON FCM_CATEGORIAAUDIT.NRCATEGORIAAUDIT = FCM_RELACMODCATEG.NRCATEGORIAAUDIT";
        
        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function getTopicosAuditoriaOffline()
    {
        $sql = "SELECT
                ROWNUM AS id,
                FCM_RELACCATTOPIC.*,
                FCM_TOPICOAUDIT.NMTOPICOAUDIT,
                FCM_OBSRESULTAUDIT.DSOBSERVACAO,
                FCM_OBSRESULTAUDIT.NROBSRESULTAUDIT,
                FCM_AUDITORIA.NRAUDITORIA
                FROM
                    FCM_RELACCATTOPIC
                JOIN FCM_AUDITORIA ON FCM_AUDITORIA.NRMODELOAUDIT = FCM_RELACCATTOPIC.NRMODELOAUDIT
                JOIN FCM_TOPICOAUDIT ON FCM_TOPICOAUDIT.NRTOPICOAUDIT = FCM_RELACCATTOPIC.NRTOPICOAUDIT
                LEFT JOIN FCM_OBSRESULTAUDIT ON FCM_OBSRESULTAUDIT.NRRELACCATTOPIC = FCM_RELACCATTOPIC.NRRELACCATTOPIC AND FCM_OBSRESULTAUDIT.NRAUDITORIA = FCM_AUDITORIA.NRAUDITORIA";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function getCriteriosAuditoriaOffline()
    {
        $sql = "SELECT 
                    FCM_RELACRESULTCRIT.NRRELACRESULTCRIT, 
                    FCM_RELACRESULTCRIT.NRRESULTADOAUDIT, 
                    FCM_RELACRESULTCRIT.NRCRITERIOAUDIT, 
                    FCM_RELACRESULTCRIT.IDREALIZADO, 
                    FCM_RELACRESULTCRIT.NRTOPICOAUDIT, 
                    FCM_CRITERIOAUDIT.NMCRITERIOAUDIT, 
                    FCM_RESULTADOAUDIT.NRCATEGORIAAUDIT,
                    FCM_AUDITORIA.NRAUDITORIA
                FROM 
                    FCM_RELACRESULTCRIT
                JOIN 
                    FCM_RESULTADOAUDIT
                    ON FCM_RESULTADOAUDIT.NRRESULTADOAUDIT = FCM_RELACRESULTCRIT.NRRESULTADOAUDIT
                JOIN 
                    FCM_CRITERIOAUDIT
                    ON FCM_CRITERIOAUDIT.NRCRITERIOAUDIT = FCM_RELACRESULTCRIT.NRCRITERIOAUDIT
                JOIN 
                    FCM_AUDITORIA
                    ON FCM_RESULTADOAUDIT.NRAUDITORIA = FCM_AUDITORIA.NRAUDITORIA";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }
    
    public function getAvaliacaoOffline()
    {
        $sql = "SELECT IDSATISFACAOSOLICITA, NRSOLICITACAO, ROWNUM AS ID
                FROM FCM_SOLICITACAOHISTORICO";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function getFotosSolicitacaoOffline()
    {

        $sql = "SELECT NRIMGSOLICITACAO, DSCAMINHO, NRSOLICITACAO
                  FROM FCM_IMGSOLICITACAO";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function getImagensAuditoriaOffline()
    {
        $sql = "SELECT
                    FCM_FOTORESULTAUDIT.*,
                    FCM_TOPICOAUDIT.NMTOPICOAUDIT
                FROM
                    FCM_FOTORESULTAUDIT
                JOIN FCM_TOPICOAUDIT ON FCM_TOPICOAUDIT.NRTOPICOAUDIT = FCM_FOTORESULTAUDIT.NRTOPICOAUDIT";
        
        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function getFotosVistoriasOffline()
    {

        $sql = "SELECT FCM_FOTOTAREFAGERCHK.NRFOTOTAREFAGERCHK, FCM_FOTOTAREFAGERCHK.DSFOTOTAREFAGERCHK, FCM_GERENCIAMENTOCHK.NRGERENCIAMENTOCHK, FCM_GERENCIAMENTOCHK.NMGERENCIAMENTOCHK
                    FROM FCM_FOTOTAREFAGERCHK
                    JOIN FCM_TAREFAGERCHK 
                    ON FCM_TAREFAGERCHK.NRTAREFAGERCHK = FCM_FOTOTAREFAGERCHK.NRTAREFAGERCHK
                    JOIN FCM_GERENCIAMENTOCHK 
                    ON FCM_GERENCIAMENTOCHK.NRGERENCIAMENTOCHK = FCM_TAREFAGERCHK.NRGERENCIAMENTOCHK";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function getFotosAprovacaoOffline()
    {

        $sql = "SELECT FTAGEND.NRFOTOTAREFAAGEND, FTAGEND.DSFOTOTAREFAAGEND, CHKAGEND.NRCHECKLISTAGEND, CHKAGEND.NMCHECKLISTAGEND
        FROM FCM_FOTOTAREFAAGEND FTAGEND
        JOIN FCM_TAREFAAGEND TRAGEND 
        ON TRAGEND.NRTAREFAAGEND = FTAGEND.NRTAREFAAGEND
        JOIN FCM_CHECKLISTAGEND CHKAGEND
        ON CHKAGEND.NRCHECKLISTAGEND = TRAGEND.NRCHECKLISTAGEND";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }
}