<?php

namespace FM\Factory;

use Doctrine\ORM\EntityManager;
use FM\Traits\TQuery;

class OrdensServicoFactory
{
    use TQuery;
    protected $entityManager;
    protected $connection;

    public function __construct(EntityManager $entityManager)
    {
        $this->connection       = $entityManager->getConnection();
        $this->entityManager    = $entityManager;
    }

    public function getOrdensServico($filtro)
    {
        $sql = "SELECT 
                  OSE.NRORDEMSERV,
                  TO_CHAR(OSE.DTORDEMSERV, 'DD/MM') AS DTORDEMSERV,
                  OSE.DURACAOORDEM,
                  OSE.IDSITUACAO,
                  EP.NMEQUIPAMENTO,
                  CAT.NMCATEGORIAMANUT,
                  TM.NMTIPOMANUTENCAO,
                  P.NMPRIORIDADEMANUT,
                  P.DSCORPRIORIDADE,
                  OSE.NREQUIPAMENTO,
                  OSE.NRCATEGORIAMANUT,
                  OSE.NRTIPOMANUTENCAO,
                  OSE.NRPRIORIDADEMANUT,
                  OSE.DSORDEMSERV,
                  OSE.NMORDEMOSERV,
                  OSE.NRSOLICITACAOSERV,
                  SS.DTSOLICITACAOSERV,
                  OP.NMOPERADOR,
                  EQP.NREQUIPE,
                  EQP.NMEQUIPE,
                  CLI.NRLOCALCLIENTE,
                  CLI.NMLOCALCLIENTE,
                  CLIE.NMFANTCLIE,
                  
                  -- Lista de funcionários responsáveis pela Ordem de Serviço
                  NVL((
                      SELECT LISTAGG(OPERADOR.NMOPERADOR, ', ') 
                          WITHIN GROUP (ORDER BY FCM_RESPONSAVELORDEMS.CDOPERADOR)
                      FROM FCM_RESPONSAVELORDEMS
                      JOIN OPERADOR 
                          ON FCM_RESPONSAVELORDEMS.CDOPERADOR = OPERADOR.CDOPERADOR
                      WHERE OSE.NRORDEMSERV = FCM_RESPONSAVELORDEMS.NRORDEMSERV
                  ), null) AS OPERADORRESPONSAVEL,

                 -- Lista de funcionarios externos responsaveis pela ordem de serviço
                  NVL((
                      SELECT LISTAGG(FCM_PROFISSIONALEXTERNO.NMPROFISSIONALEXTERNO, ', ') 
                          WITHIN GROUP (ORDER BY FCM_RESPONSAVELORDEMS.NRPROFISSIONALEXTERNO)
                      FROM FCM_RESPONSAVELORDEMS
                      JOIN FCM_PROFISSIONALEXTERNO 
                          ON FCM_RESPONSAVELORDEMS.NRPROFISSIONALEXTERNO = FCM_PROFISSIONALEXTERNO.NRPROFISSIONALEXTERNO
                      WHERE OSE.NRORDEMSERV = FCM_RESPONSAVELORDEMS.NRORDEMSERV
                  ), null) AS PROFISSIONALEXTERNO,

                  -- Lista de escopos de manutenção relacionados à Ordem de Serviço
                  NVL((
                      SELECT LISTAGG(FCM_ESCOPOMANUT.NMESCOPOMANUT, ', ') 
                          WITHIN GROUP (ORDER BY FCM_ESCOPOMANUT.NRESCOPOMANUT)
                      FROM FCM_RELESCORDEMSERV
                      JOIN FCM_ESCOPOMANUT 
                          ON FCM_RELESCORDEMSERV.NRESCOPOMANUT = FCM_ESCOPOMANUT.NRESCOPOMANUT
                      WHERE OSE.NRORDEMSERV = FCM_RELESCORDEMSERV.NRORDEMSERV
                  ), null) AS NMESCOPOMANUT,

                  -- Descrição do status da Ordem de Serviço
                  CASE OSE.IDSITUACAO 
                      WHEN '1' THEN 'Pendente'
                      WHEN '2' THEN 'Em Andamento'
                      WHEN '3' THEN 'Finalizada'
                      WHEN '4' THEN 'Recusada'
                      WHEN '5' THEN 'Pausada'
                      ELSE NULL 
                  END AS NMIDSITUACAO

              FROM FCM_ORDEMSERV OSE

              -- Junções com outras tabelas
              LEFT JOIN FCM_SOLICITACAOSERV SS 
                  ON OSE.NRSOLICITACAOSERV = SS.NRSOLICITACAOSERV

              LEFT JOIN FCM_PRIORIDADEMANUT P 
                  ON P.NRPRIORIDADEMANUT = OSE.NRPRIORIDADEMANUT

              LEFT JOIN FCM_EQUIPE EQP 
                  ON EQP.NREQUIPE = OSE.NREQUIPE

              LEFT JOIN FCM_EQUIPAMENTO EP 
                  ON EP.NREQUIPAMENTO = OSE.NREQUIPAMENTO

              LEFT JOIN FCM_CATEGORIAMANUT CAT 
                  ON CAT.NRCATEGORIAMANUT = OSE.NRCATEGORIAMANUT

              LEFT JOIN FCM_TIPOMANUTENCAO TM 
                  ON TM.NRTIPOMANUTENCAO = OSE.NRTIPOMANUTENCAO

              LEFT JOIN OPERADOR OP 
                  ON SS.CDOPERSOLICITANTE = OP.CDOPERADOR

              LEFT JOIN FCM_LOCALCLI CLI
                  ON CLI.NRLOCALCLIENTE = OSE.NRLOCALCLIENTE

              LEFT JOIN CLIENTE CLIE
                  ON CLIE.CDCLIENTE = CLI.CDCLIENTE

              LEFT JOIN FCM_RESPONSAVELORDEMS FR
                  ON FR.NRORDEMSERV = OSE.NRORDEMSERV


              WHERE OSE.NRORDEMSERV IS NOT NULL

                " . $filtro . " ORDER BY DTORDEMSERV ASC";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }
    
    public function getImagensOrdemServ($parametros) 
    {
        $sql = "SELECT NRIMGORDEMSERV, NRORDEMSERV, DSCAMINHO, DTINCLUSAO 
                FROM FCM_IMGORDEMSERV
                WHERE NRORDEMSERV = :NRORDEMSERV";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function finalizarOS($parametros)
    {
      $sql = "UPDATE FCM_ORDEMSERV
              SET    IDSITUACAO = 3,
                     DURACAOORDEM = DURACAOORDEM + NUMTODSINTERVAL((SYSDATE - DTULTATU) * 24 * 60 * 60, 'SECOND'),
                     DTULTATU = SYSDATE,
                     CDOPERULTATU = :CDOPERADOR,
                     NRORGULTATU = :NRORG
              WHERE  NRORDEMSERV = :NRORDEMSERV";

      $this->connection->executeStatement($sql, $parametros);
    }

    public function recusarOS($parametros)
    {
      $sql = "UPDATE FCM_ORDEMSERV
              SET    IDSITUACAO = 4,
                     DTULTATU = SYSDATE,
                     CDOPERULTATU = :CDOPERADOR,
                     NRORGULTATU = :NRORG
              WHERE  NRORDEMSERV = :NRORDEMSERV";

      $this->connection->executeStatement($sql, $parametros);
    }

    public function insertHistoricoOS($parametros)
    {
        $sql = "INSERT INTO FCM_ORDEMSERVHISTORICO(
                    NRORDEMSERVHISTORICO,   NRSOLICITACAOSERV,  NRORDEMSERV, 
                    DTHISTORDEMSERV,        IDATIVO,            DSORDEMSERVHISTORICO, 
                    NRORG,                  DTINCLUSAO,         CDOPERINCLUSAO, 
                    NRORGINCLUSAO
                )
                VALUES(
                    :NRORDEMSERVHISTORICO,   :NRSOLICITACAOSERV, :NRORDEMSERV, 
                    SYSDATE,                 'S',                :DSORDEMSERVHISTORICO, 
                    :NRORG,                   SYSDATE,           :CDOPERINCLUSAO, 
                    :NRORG
                )";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function iniciarOS($parametros)
    {
        $sql = "UPDATE FCM_ORDEMSERV
              SET    IDSITUACAO = 2,
                     DTULTATU = SYSDATE,
                     CDOPERULTATU = :CDOPERADOR,
                     NRORGULTATU = :NRORG
              WHERE  NRORDEMSERV = :NRORDEMSERV";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function pausarOS($parametros)
    {
        $sql = "UPDATE FCM_ORDEMSERV
                SET IDSITUACAO = 5,
                    DURACAOORDEM = DURACAOORDEM + NUMTODSINTERVAL((SYSDATE - DTULTATU) * 24 * 60 * 60, 'SECOND'),
                    DTULTATU = SYSDATE,
                    CDOPERULTATU = :CDOPERADOR,
                    NRORGULTATU = :NRORG
                WHERE NRORDEMSERV = :NRORDEMSERV";
              
        $this->connection->executeStatement($sql, $parametros);
    }

    public function getModeloPopOrdemServ($parametros)
    {
        $sql = "SELECT FCM_MODELOPOP.NRMODELOPOP, FCM_MODELOPOP.NMMODELOPOP, FCM_MODELOPOP.DESCMODELOPOP, 
                    FCM_MODELOPOP.LINKMODELOPOP, FCM_ANEXOMODELOPOP.DSCAMINHOARQ, 
                    FCM_ANEXOMODELOPOP.NRANEXOMODELOPOP
                    FROM FCM_ORDEMSERV
                    JOIN FCM_MODELOPOP
                    ON FCM_MODELOPOP.NRMODELOPOP = FCM_ORDEMSERV.NRMODELOPOP
                    JOIN FCM_ANEXOMODELOPOP
                    ON FCM_ANEXOMODELOPOP.NRMODELOPOP = FCM_MODELOPOP.NRMODELOPOP
                    WHERE FCM_ORDEMSERV.NRORDEMSERV = :NRORDEMSERV";
        
        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function getAnexoModeloPopOrdemServ($parametros)
    {
        $sql = "SELECT NMARQUIVO, DSCAMINHOARQ, DSTIPO, VRTAMANHO 
                FROM FCM_ANEXOMODELOPOP
                WHERE FCM_ANEXOMODELOPOP.NRANEXOMODELOPOP = :NRANEXOMODELOPOP";
        
        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

    public function getEpisOS($parametros)
    {
        $sql = "SELECT FCM_EPI.NREPI, FCM_EPI.NMEPI
                FROM FCM_RELACEPIORDEMS
                JOIN FCM_EPI
                ON FCM_EPI.NREPI = FCM_RELACEPIORDEMS.NREPI
                WHERE FCM_RELACEPIORDEMS.NRORDEMSERV = :NRORDEMSERV";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
    }

<<<<<<< HEAD
    public function getTipoLocalTarefa($cliente)
    {

        $sql = "SELECT DISTINCT FCM_TIPOLOCALCLI.NRTIPOLOCALCLI, FCM_TIPOLOCALCLI.NMTIPOLOCALCLI
                FROM FCM_TIPOLOCALCLI
                JOIN FCM_LOCALCLI
                ON FCM_LOCALCLI.NRTIPOLOCALCLI = FCM_TIPOLOCALCLI.NRTIPOLOCALCLI
                WHERE FCM_TIPOLOCALCLI.IDTIPOLOCALCLI = '3'
                AND FCM_TIPOLOCALCLI.IDATIVO = 'S'
                AND FCM_LOCALCLI.CDCLIENTE IN ($cliente)
                ORDER BY NMTIPOLOCALCLI";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function getLocal($param, $cliente)
    {

        $sql = "SELECT FCM_LOCALCLI.NMLOCALCLIENTE, FCM_LOCALCLI.NRLOCALCLIENTE, FCM_LOCALCLI.NRESTRUTURAM, FCM_LOCALCLI.NRTIPOLOCALCLI, FCM_LOCALCLI.NRLOCALCLIENTEPAI, LOCP.NMLOCALCLIENTE AS NMLOCALCLIENTEPAI
                FROM FCM_LOCALCLI
                JOIN FCM_LOCALCLI LOCP ON LOCP.NRLOCALCLIENTE = FCM_LOCALCLI.NRLOCALCLIENTEPAI
                JOIN FCM_TIPOLOCALCLI ON FCM_TIPOLOCALCLI.NRTIPOLOCALCLI = FCM_LOCALCLI.NRTIPOLOCALCLI
                WHERE FCM_LOCALCLI.IDATIVO = 'S'
                AND FCM_LOCALCLI.NRTIPOLOCALCLI IN ($param)
                AND FCM_LOCALCLI.CDCLIENTE = $cliente";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function insertOrdemServico($parametros)
    {
        $sql = "INSERT INTO FCM_ORDEMSERV(
                    NRORDEMSERV,        NMORDEMOSERV,       IDSITUACAO,     NREQUIPAMENTO,  NRCATEGORIAMANUT,
                    NRTIPOMANUTENCAO,   NRPRIORIDADEMANUT,  DSORDEMSERV,    DTINCLUSAO,    
                    NRORG,              DTORDEMSERV,        NRSOLICITACAOSERV, NRLOCALCLIENTE, NRMODELOPOP, NREQUIPE, IDTAREFARESPONSAVEL
                )
                VALUES(
                    :NRORDEMSERV,       :NMORDEMOSERV,       :IDSITUACAO,    :NREQUIPAMENTO, :NRCATEGORIAMANUT,
                    :NRTIPOMANUTENCAO,  :NRPRIORIDADEMANUT, :DSORDEMSERV,   SYSDATE,        
                    :NRORG,             TO_DATE(:DTORDEMSERV, 'DD/MM/YYYY'), :NRSOLICITACAOSERV, :NRLOCALCLIENTE, :NRMODELOPOP, :NREQUIPE, :IDTAREFARESPONSAVEL
                )";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function insertRelSolicitacaoOrdem($parametros)
    {
        $sql = "INSERT INTO FCM_RELSOLICITAORDEMSERV(
                    NRRELSOLICITAORDEMSERV,     NRSOLICITACAOSERV,  NRORDEMSERV,
                    IDATIVO,                    DTINCLUSAO
                )
                VALUES(
                    :NRRELSOLICITAORDEMSERV,    :NRSOLICITACAOSERV, :NRORDEMSERV,
                    'S',                        SYSDATE
                )";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function alterarStatusSolicitacao($parametros)
    {
        $sql = "UPDATE FCM_SOLICITACAOSERV
                    SET IDSITUACAO = :IDSITUACAO
=======
    public function finalizarSS($parametros)
    {
        $sql = "UPDATE FCM_SOLICITACAOSERV
                SET IDSITUACAO = '4'
                    DTULTATU = SYSDATE,
                    CDOPERULTATU = :CDOPERADOR,
                    NRORGULTATU = :NRORG
>>>>>>> 532949956caf2aed72989bdcb5024e83383d36e8
                WHERE NRSOLICITACAOSERV = :NRSOLICITACAOSERV";

        $this->connection->executeStatement($sql, $parametros);
    }
<<<<<<< HEAD

    public function insertRelEscopoOrdem($parametros)
    {
        $sql = "INSERT INTO FCM_RELESCORDEMSERV(
                    NRRELESCORDEMSERV,  NRORDEMSERV,    NRESCOPOMANUT,  DTINCLUSAO
                )
                VALUES(
                    :NRRELESCORDEMSERV, :NRORDEMSERV,   :NRESCOPOMANUT, SYSDATE
                )";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function insertRelacaoResponsavelOrdem($parametros)
    {
        $sql = "INSERT INTO FCM_RESPONSAVELORDEMS (NRRESPONSAVELORDEMS, NRORDEMSERV, NRPROFISSIONALEXTERNO, CDOPERADOR, IDATIVO, NRORG, DTINCLUSAO, CDOPERINCLUSAO, NRORGINCLUSAO)
                VALUES (:NRRESPONSAVELORDEMS, :NRORDEMSERV, :NRPROFISSIONALEXTERNO, :CDOPERADOR, 'S',  :NRORG, SYSDATE, :CDOPERINCLUSAO, :NRORGINCLUSAO)";

        return $this->connection->executeStatement($sql, $parametros);
    }

    public function insertRelacaoEpis($parametros)
    {
        $sql = "INSERT INTO FCM_RELACEPIORDEMS (NRRELACEPIORDEMS, NREPI, NRORDEMSERV, IDATIVO, NRORG, DTINCLUSAO, CDOPERINCLUSAO, NRORGINCLUSAO)
                VALUES (:NRRELACEPIORDEMS, :NREPI, :NRORDEMSERV, 'S',  :NRORG, SYSDATE, :CDOPERINCLUSAO, :NRORGINCLUSAO)";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function insertFotoOrdem($parametros)
    {
        $sql = "INSERT INTO FCM_IMGORDEMSERV (
                    NRIMGORDEMSERV,     NRORDEMSERV,    DSCAMINHO,  NRORG,
                    DTINCLUSAO,         DSTYPE,         NRSIZE
                )
                VALUES (
                    :NRIMGORDEMSERV,    :NRORDEMSERV,   :DSCAMINHO, :NRORG,
                    SYSDATE,            :DSTYPE,        :NRSIZE
                )";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function getEscopos()
    {
        $sql = "SELECT NRESCOPOMANUT, NMESCOPOMANUT
				FROM FCM_ESCOPOMANUT";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }
=======
>>>>>>> 532949956caf2aed72989bdcb5024e83383d36e8
}