<?php

namespace FM\Service;

use FM\Factory\OrdensServicoFactory;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;
use FM\Service\FindService;
use FM\Service\FileServerService;

class OrdensServicoService
{

    protected $OrdensServicoFactory;
    protected $environment;
    protected $newCode;
    protected $connection;
    protected $FuncionariosHierarqService;
    protected $FindService;
    protected $FileServerService;
    protected $fileServerUrl;

    public function __construct(OrdensServicoFactory $OrdensServicoFactory, EntityManager $entityManager, Environment $environment, NewCode $newCode, FuncionariosHierarqService $FuncionariosHierarqService, FindService $FindService, FileServerService $FileServerService, $fileServerUrl)
    {
        $this->connection                   = $entityManager->getConnection();
        $this->OrdensServicoFactory         = $OrdensServicoFactory;
        $this->environment                  = $environment;
        $this->newCode                      = $newCode;
        $this->FuncionariosHierarqService   = $FuncionariosHierarqService;
        $this->FindService                  = $FindService;
        $this->FileServerService            = $FileServerService;
        $this->fileServerUrl                = $fileServerUrl;
    }

    public function getOrdensServico($parametros)
    {
        $filtro = $this->montarFiltros($parametros);
        return $this->OrdensServicoFactory->getOrdensServico($filtro);
    }

    public function getImagensOrdemServ($parametros)
    {
        return $this->OrdensServicoFactory->getImagensOrdemServ($parametros);
    }

    public function finalizarOS($parametros)
    {
        try {
            $parametros["CDOPERADOR"] = $this->environment->getCdOperador();
            $parametros["NRORG"] = $this->environment->getNrOrg();
            $this->OrdensServicoFactory->finalizarOS($parametros);

            if (!empty($parametros['NRSOLICITACAOSERV'])) {
                $this->finalizarSS($parametros);
            }

            $parametros["DSORDEMSERVHISTORICO"] = "Ordem de serviço finalizada";
            $this->insertHistoricoOS($parametros);
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function recusarOS($parametros)
    {
        try {
            $parametros["CDOPERADOR"] = $this->environment->getCdOperador();
            $parametros["NRORG"] = $this->environment->getNrOrg();
            $this->OrdensServicoFactory->recusarOS($parametros);

            $parametros["DSORDEMSERVHISTORICO"] = "Ordem de serviço recusada. Motivo: " . $parametros["MOTIVO"];
            $this->insertHistoricoOS($parametros);
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function iniciarOS($parametros)
    {
        try {
            $parametros["CDOPERADOR"] = $this->environment->getCdOperador();
            $parametros["NRORG"] = $this->environment->getNrOrg();
            $this->OrdensServicoFactory->iniciarOS($parametros);

            $parametros["DSORDEMSERVHISTORICO"] = "Ordem de serviço iniciada.";
            $this->insertHistoricoOS($parametros);
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function pausarOS($parametros)
    {
        try {
            $parametros["CDOPERADOR"] = $this->environment->getCdOperador();
            $parametros["NRORG"] = $this->environment->getNrOrg();
            $this->OrdensServicoFactory->pausarOS($parametros);

            $parametros["DSORDEMSERVHISTORICO"] = "Ordem de serviço pausada.";
            $this->insertHistoricoOS($parametros);
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }

    }

    public function getModeloPopOrdemServ($parametros)
    {
        return $this->OrdensServicoFactory->getModeloPopOrdemServ(array(
            "NRORDEMSERV"   => $parametros["NRORDEMSERV"],
        ));
    }

    public function getAnexoModeloPopOrdemServ($parametros)
    {
        return $this->OrdensServicoFactory->getAnexoModeloPopOrdemServ(array(
            "NRORDEMSERV"   => $parametros["NRORDEMSERV"],
        ));
    }

    public function getEpisOS($parametros)
    {
        return $this->OrdensServicoFactory->getEpisOS(array(
            "NRORDEMSERV"   => $parametros["NRORDEMSERV"],
        ));
    }

    public function montarFiltros($parametros)
    {
        $filtro = '';

        if (isset($parametros['NRPRIORIDADEMANUT'])) {
            $prioridade = is_array($parametros['NRPRIORIDADEMANUT']) ? $this->FindService->formatarArray($parametros['NRPRIORIDADEMANUT']) : "'" . $parametros['NRPRIORIDADEMANUT'] . "'";
            $filtro .= " AND OSE.NRPRIORIDADEMANUT IN ( " . $prioridade . ")";
        }

        if (isset($parametros['NRTIPOMANUTENCAO'])) {
            $tipoManut = is_array($parametros['NRTIPOMANUTENCAO']) ? $this->FindService->formatarArray($parametros['NRTIPOMANUTENCAO']) : "'" . $parametros['NRTIPOMANUTENCAO'] . "'";
            $filtro .= " AND OSE.NRTIPOMANUTENCAO IN ( " . $tipoManut . ")";
        }

        if (isset($parametros['NREQUIPAMENTO'])) {
            $equipamento = is_array($parametros['NREQUIPAMENTO']) ? $this->FindService->formatarArray($parametros['NREQUIPAMENTO']) : "'" . $parametros['NREQUIPAMENTO'] . "'";
            $filtro .= " AND OSE.NREQUIPAMENTO IN ( " . $equipamento . ")";
        }

        if (isset($parametros['NRCATEGORIAMANUT'])) {
            $categoriaManut = is_array($parametros['NRCATEGORIAMANUT']) ? $this->FindService->formatarArray($parametros['NRCATEGORIAMANUT']) : "'" . $parametros['NRCATEGORIAMANUT'] . "'";
            $filtro .= " AND OSE.NRCATEGORIAMANUT IN ( " . $categoriaManut . ")";
        }

        if (isset($parametros['IDSITUACAO'])) {
            $situacao = is_array($parametros['IDSITUACAO']) ? $this->FindService->formatarArray($parametros['IDSITUACAO']) : "'" . $parametros['IDSITUACAO'] . "'";
            $filtro .= " AND OSE.IDSITUACAO IN ( " . $situacao . ")";
        }

        if (isset($parametros['IDATIVO'])) {
            $filtro .= " AND OSE.IDATIVO = '" . $parametros['IDATIVO'] . "'";
        }

        if (isset($parametros['NREQUIPE'])) {
            $situacao = is_array($parametros['NREQUIPE']) ? $this->FindService->formatarArray($parametros['NREQUIPE']) : "'" . $parametros['NREQUIPE'] . "'";
            $filtro .= " AND OSE.NREQUIPE IN ( " . $situacao . ")";
        }

        if (isset($parametros['CDOPERADOR'])) {
            $situacao = is_array($parametros['CDOPERADOR']) ? $this->FindService->formatarArray($parametros['CDOPERADOR']) : "'" . $parametros['CDOPERADOR'] . "'";
            $filtro .= " AND FR.CDOPERADOR IN ( " . $situacao . ")";
        }

        if (isset($parametros['NRPROFISSIONALEXTERNO'])) {
            $situacao = is_array($parametros['NRPROFISSIONALEXTERNO']) ? $this->FindService->formatarArray($parametros['NRPROFISSIONALEXTERNO']) : "'" . $parametros['NRPROFISSIONALEXTERNO'] . "'";
            $filtro .= " AND FR.NRPROFISSIONALEXTERNO IN ( " . $situacao . ")";
        }

        return $filtro;
    }


    public function insertHistoricoOS($parametros) {
        try {
            $this->connection->beginTransaction();
            $nrorg = $this->environment->getNrorg();

            $this->OrdensServicoFactory->insertHistoricoOS(array(
                "NRORDEMSERVHISTORICO" => $this->newCode->getCodeByTableName("FCM_ORDEMSERVHISTORICO", $nrorg, 1, 10, true),
                "NRSOLICITACAOSERV"    => $parametros["NRSOLICITACAOSERV"] ?? null,
                "NRORDEMSERV"          => $parametros["NRORDEMSERV"],
                "DSORDEMSERVHISTORICO" => $parametros["DSORDEMSERVHISTORICO"],
                "NRORG"                => $nrorg,
                "CDOPERINCLUSAO"       => $this->environment->getCdOperador(),
            ));

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
<<<<<<< HEAD
=======
        }
    }

    public function finalizarSS($parametros)
    {
        try {
            $this->connection->beginTransaction();

            $this->OrdensServicoFactory->finalizarSS(array(
                "NRSOLICITACAOSERV" => $parametros["NRSOLICITACAOSERV"],
                "NRORG"             => $this->environment->getNrorg(),
                "CDOPERADOR"        => $this->environment->getCdOperador(),
            ));
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
>>>>>>> 532949956caf2aed72989bdcb5024e83383d36e8
        }
    }

    public function getTipoLocalTarefa($parametros)
    {
        $arrayClientes = (is_array($parametros['CDCLIENTE'])) ? $this->FindService->formatarArray($parametros['CDCLIENTE']) : "'" . $parametros['CDCLIENTE'] . "'";
        return $this->OrdensServicoFactory->getTipoLocalTarefa($arrayClientes);
    }

    public function getLocal($parametros)
    {
        $tipoLocal = ($parametros['UNICA'] == "S") ? $parametros['NRTIPOLOCALCLI'] : $this->FindService->formatarArray($parametros['NRTIPOLOCALCLI']);
        return $this->OrdensServicoFactory->getLocal($tipoLocal, "'" . $parametros['CDCLIENTE'] . "'");
    }

    public function criarOrdemServico($parametros)
    {
        $this->connection->beginTransaction();
        try {
            $nrorg = $this->environment->getNrorg();
            $NRORDEMSERV = $this->newCode->getCodeByTableName("FCM_ORDEMSERV", $nrorg, 1, 10, true);

            $DSORDEMSERVHISTORICO = "Ordem de serviço criada";

            $parametroOrdem = array(
                "NRORDEMSERV"           => $NRORDEMSERV,
                "NMORDEMOSERV"          => $parametros["NMORDEMOSERV"],
                "NREQUIPAMENTO"         => $parametros["NREQUIPAMENTO"],
                "NRCATEGORIAMANUT"      => $parametros["NRCATEGORIAMANUT"],
                "NRTIPOMANUTENCAO"      => $parametros["NRTIPOMANUTENCAO"],
                "NRPRIORIDADEMANUT"     => $parametros["NRPRIORIDADEMANUT"],
                "DTORDEMSERV"           => $parametros["DTORDEMSERV"],
                "IDSITUACAO"            => $parametros["IDSITUACAO"],
                "DSORDEMSERV"           => $parametros["DSORDEMSERV"],
                "NRSOLICITACAOSERV"     => $parametros["NRSOLICITACAOSERV"] ?? null,
                "NRORG"                 => $nrorg,
                "NRLOCALCLIENTE"        => $parametros['NRLOCALCLIENTE'],
                "NRMODELOPOP"           => $parametros['NRMODELOPOP'],
                "NREQUIPE"              => $parametros['NREQUIPE'],
                "IDTAREFARESPONSAVEL"   => (!empty($parametros['NREQUIPE'])) ? 'N' : 'S'
            );

            $this->OrdensServicoFactory->insertOrdemServico($parametroOrdem);

            $this->insertRelacaoResponsavelOrdem($parametros, $NRORDEMSERV);

            $this->insertHistoricoOS(array(
                "NRORDEMSERV"           => $NRORDEMSERV,
                "NRSOLICITACAOSERV"     => $parametros["NRSOLICITACAOSERV"] ?? null,
                "DSORDEMSERVHISTORICO"  => $DSORDEMSERVHISTORICO,
            ));
        
            if (count ($parametros["NRESCOPOS"]) > 0) {
                foreach ($parametros["NRESCOPOS"] as $NRESCOPOMANUT) {
                    $NRRELESCORDEMSERV = $this->newCode->getCodeByTableName("FCM_RELESCORDEMSERV", $nrorg, 1, 10, true);
    
                    $parametroEscopo = array(
                        "NRRELESCORDEMSERV" => $NRRELESCORDEMSERV,
                        "NRORDEMSERV"       => $NRORDEMSERV,
                        "NRESCOPOMANUT"     => $NRESCOPOMANUT
                    );
    
                    $this->OrdensServicoFactory->insertRelEscopoOrdem($parametroEscopo);
                }
            }

            if (count($parametros['NREPI']) > 0) {
                $this->insertRelacaoEpis($parametros, $NRORDEMSERV);
            }
    
            if (isset($parametros["FOTOS"])) {
                $parametroFotos = array(
                    "NRORDEMSERV"   => $NRORDEMSERV,
                    "FOTOS"         => $parametros["FOTOS"]
                );
                $this->insertFotosOrdem($parametroFotos);
            }

            $this->connection->commit();
        } catch (\Exception $ex) {
            $this->connection->rollback();
            throw $ex;
        }
    }

    public function insertRelacaoResponsavelOrdem($parametros, $NRORDEMSERV)
    {
        $arrayResponsavel = [];
        $responsavel = '';
        $nrorg = $this->environment->getNrorg();
        $operadorLogado = $this->environment->getCdOperador();
        $profissionalExterno = array();

        if ($parametros['NRPROFISSIONALEXTERNO'] !== null) {
            array_push($profissionalExterno, $parametros['NRPROFISSIONALEXTERNO']);
        }

        // if (!empty($parametros['NOVOPROFISSIONAL'])) {
        //     $nrprofissionalExterno = $this->insertProfissionalExterno($parametros['NOVOPROFISSIONAL']);
        //     array_push($profissionalExterno, $nrprofissionalExterno);
        // }

        if (!empty($parametros['CDOPERADOR'])) {
            $arrayResponsavel = $parametros['CDOPERADOR'];
            $responsavel = 'operador';
        } else {
            $arrayResponsavel = $profissionalExterno;
            $responsavel = 'externo';
        }

        $this->connection->beginTransaction();

        try {

            foreach ($arrayResponsavel as $resp) {
                $NRRESPONSAVELORDEMS = $this->newCode->getCodeByTableName("FCM_RESPONSAVELORDEMS", $nrorg, 1, 10, true);
                
                $params = array(
                    "NRRESPONSAVELORDEMS"       => $NRRESPONSAVELORDEMS,
                    "NRORDEMSERV"               => $NRORDEMSERV,
                    "NRPROFISSIONALEXTERNO"     => ($responsavel === 'externo') ? $resp : null,
                    "CDOPERADOR"                => ($responsavel === 'operador') ? $resp : null,
                    "CDOPERINCLUSAO"            => $operadorLogado,
                    "NRORG"                     => $nrorg,
                    "NRORGINCLUSAO"             => $nrorg,
                );

                $this->OrdensServicoFactory->insertRelacaoResponsavelOrdem($params);
            }
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function insertRelacaoEpis($parametros, $NRORDEMSERV)
    {
        $this->connection->beginTransaction();

        try {
            $nrorg = $this->environment->getNrorg();
            $operadorLogado = $this->environment->getCdOperador();
        
            foreach ($parametros["NREPI"] as $EPI) {
                $NRRELACEPIORDEMS = $this->newCode->getCodeByTableName("FCM_RELACEPIORDEMS", $nrorg, 1, 10, true);

                $this->OrdensServicoFactory->insertRelacaoEpis(array(
                    "NRRELACEPIORDEMS"  => $NRRELACEPIORDEMS,
                    "NRORDEMSERV"       => $NRORDEMSERV,
                    "NREPI"             => $EPI,
                    "CDOPERINCLUSAO"    => $operadorLogado,
                    "NRORGINCLUSAO"     => $nrorg,
                    "NRORG"             => $nrorg,
                ));
            }

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function insertFotosOrdem($parametros)
    {
        $this->connection->beginTransaction();

        try {
            $nrorg = $this->environment->getNrorg();

            foreach ($parametros["FOTOS"] as $foto) {
                $img = $this->FileServerService->uploadFoto([$foto], "foto_manutencao_fm");

                $NRIMGORDEMSERV = $this->newCode->getCodeByTableName('FCM_IMGORDEMSERV', $nrorg, 1, 10, true);
                $parametroFoto = array(
                    "NRIMGORDEMSERV" => $NRIMGORDEMSERV,
                    "NRORDEMSERV" => $parametros["NRORDEMSERV"],
                    "DSCAMINHO" => $this->fileServerUrl . "/files/" . $img["path"],
                    "NRORG" => $nrorg,
                    "DSTYPE" => $img["type"],
                    "NRSIZE" => $img["size"]
                );

                $this->OrdensServicoFactory->insertFotoOrdem($parametroFoto);


                $this->insertHistoricoOS(array(
                    "NRORDEMSERV" => $parametros["NRORDEMSERV"],
                    "DSORDEMSERVHISTORICO" => "Foto adicionada à ordem de serviço",
                ));
            }
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function getEscopos()
    {
        return $this->OrdensServicoFactory->getEscopos();
    }
}