<?php

namespace FM\Service;

use FM\Factory\SolicitacaoServFactory;
use Doctrine\ORM\EntityManager;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;

class SolicitacaoServService
{
    protected $SolicitacaoServFactory;
    protected $FileServerService;
    protected $connection;
    protected $Environment;
    protected $newCode;
    protected $fileServerUrl;

    public function __construct(SolicitacaoServFactory $SolicitacaoServFactory, FileServerService $FileServerService, EntityManager $entityManager, Environment $Environment, NewCode $newCode, $fileServerUrl)
    {
        $this->SolicitacaoServFactory = $SolicitacaoServFactory;
        $this->FileServerService = $FileServerService;
        $this->connection = $entityManager->getConnection();
        $this->Environment = $Environment;
        $this->newCode = $newCode;
        $this->fileServerUrl = $fileServerUrl;
    }

    public function getSolicitacoesServDash()
    {
        return $this->SolicitacaoServFactory->getSolicitacoesServDash();
    }

    public function getSolicitacoesAbertas()
    {
        return $this->SolicitacaoServFactory->getSolicitacoesAbertas();
    }

    public function getSolicitacoeConvertidasOS()
    {
        return $this->SolicitacaoServFactory->getSolicitacoeConvertidasOS();
    }

    public function saveSolicitacaoServico($parametros)
    {
        $this->connection->beginTransaction();

        try {
            $nrorg = $this->Environment->getNrorg();
            $operador = $this->Environment->getCdOperador();

            $NRSOLICITACAOSERV = $this->newCode->getCodeByTableName("FCM_SOLICITACAOSERV", $nrorg, 1, 10, true);
            $parametrosSolicitacao = array(
                "NRSOLICITACAOSERV"     => $NRSOLICITACAOSERV,
                "NREQUIPAMENTO"         => $parametros["NREQUIPAMENTO"],
                "NRCATEGORIAMANUT"      => $parametros["NRCATEGORIAMANUT"],
                "NRIMPACTOMANUT"        => $parametros["NRIMPACTOMANUT"],
                "DSSOLICITACAOSERV"     => $parametros["DSSOLICITACAOSERV"],
                "NRORG"                 => $nrorg,
                "CDOPERINCLUSAO"        => $operador,
                "NRORGINCLUSAO"         => $nrorg,
                "CDOPERULTATU"          => $operador,
                "NRORGULTATU"           => $nrorg,
                "NRGERENCIAMENTOCHK"    => (!empty($parametros["NRGERENCIAMENTOCHK"])) ? $parametros["NRGERENCIAMENTOCHK"] : null,
                "NRCHECKLISTAGEND"      => (!empty($parametros["NRCHECKLISTAGEND"])) ? $parametros["NRCHECKLISTAGEND"] : null,
                "NMSOLICITACAOSERV"     => $parametros["NMSOLICITACAOSERV"],
                "CDOPERSOLICITANTE"     => $parametros["CDOPERSOLICITANTE"],
                "NRLOCALCLIENTE"        => $parametros["NRLOCALCLIENTE"]
            );
            $this->SolicitacaoServFactory->saveSolicitacaoServico($parametrosSolicitacao);

            if (!empty($parametros["FOTOS"])) {

                $parametroFoto = array(
                    "NRSOLICITACAOSERV" => $NRSOLICITACAOSERV,
                    "FOTOS"             => $parametros["FOTOS"]
                );
                $this->insertFotosSolicitacao($parametroFoto);
            }

            $this->insertHistorico(array(
                "NRSOLICITACAOSERV" => $NRSOLICITACAOSERV,
                "DSORDEMSERVHISTORICO"  => 'Foi gerada uma nova solicitação de serviço.',
            ));

            $this->connection->commit();
        } catch (\Exception $ex) {
            $this->connection->rollback();
        }
    }

    public function insertFotosSolicitacao($parametros)
    {
        $this->connection->beginTransaction();

        try {
            $nrorg = $this->Environment->getNrorg();

            foreach ($parametros["FOTOS"] as $foto) {
                $img = $this->FileServerService->uploadFoto([$foto], "foto_vistoria_fm");

                $NRIMGSOLICITACAOSERV = $this->newCode->getCodeByTableName('FCM_IMGSOLICITACAOSERV', 20, 1, 10, true);
                $parametroFoto = array(
                    "NRIMGSOLICITACAOSERV"  => $NRIMGSOLICITACAOSERV,
                    "NRSOLICITACAOSERV"     => $parametros["NRSOLICITACAOSERV"],
                    "DSCAMINHO"             => $this->fileServerUrl . "/files/" . $img["path"],
                    "NRORG"                 => $nrorg,
                    "DSTYPE"                => $img["type"],
                    "NRSIZE"                => $img["size"]
                );

                $this->SolicitacaoServFactory->insertFotoSolicitacao($parametroFoto);
            }

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
        }
    }

    public function insertHistorico($param)
    {
        $this->connection->beginTransaction();
        try {
            $nrorg = $this->Environment->getNrorg();

            $this->SolicitacaoServFactory->insertHistorico(array(
                "NRORDEMSERVHISTORICO" => $this->newCode->getCodeByTableName("FCM_ORDEMSERVHISTORICO", $nrorg, 1, 10, true),
                "NRSOLICITACAOSERV"    => $param["NRSOLICITACAOSERV"] ?? null,
                "NRORDEMSERV"          => $param["NRORDEMSERV"] ?? null,
                "DSORDEMSERVHISTORICO" => $param["DSORDEMSERVHISTORICO"],
                "NRORG"                => $nrorg,
                "CDOPERINCLUSAO"       => $this->Environment->getCdOperador(),
            ));

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
        }
    }
}
