import { HttpCRUDTranslator } from '@/http-crud-translator/http-crud-translator';
import { DashboardResources } from '@/server/config/Resources';
import { IterableComponentRender } from '@zeedhi/common';
import {
    IDictionary,
    IEventParam,
} from '@zeedhi/core';
import { ECharts } from '@zeedhi/zd-echarts';
import { FiltroUtil } from './filtroUtil';

export class DashboardController {
    public async dataBaseChartPie({ component }: { component: ECharts }) {
        await HttpCRUDTranslator.get('/dataBaseChartPie').then((response: IDictionary) => {
            const newOptions = { ...component.options };
            if (response.data.data.length) {
                response.data.data.forEach((dados: IDictionary) => {
                    const dado = {
                        name: dados.NMATIVIDADE,
                        value: dados.QUANTIDADE,
                        itemStyle: {
                            color: dados.DSCORIDATIVIDADE,
                        },
                    };
                    newOptions.series[0].data.push(dado);
                });
                component.setOption(newOptions);
            } else {
                const dado = {
                    name: 'Não há dados',
                    value: 100,
                    itemStyle: {
                        color: '#57737a',
                    },
                };
                newOptions.series[0].data.push(dado);
                component.setOption(newOptions);
            }
        });
    }

    public async dataBaseChartBar({ component }: { component: ECharts }) {
        HttpCRUDTranslator.get('/dataBaseChartBar').then((response: IDictionary) => {
            const newOptions = { ...component.options };
            if (response.data.data.length) {
                response.data.data.forEach((dados: IDictionary) => {
                    newOptions.xAxis.data.push(dados.NMCRITERIOTARGERCHK);
                    const dado = {
                        value: dados.QTDADE,
                        itemStyle: {
                            color: dados.DSCORIDATIVIDADE,
                        },
                    };
                    newOptions.series[0].data.push(dado);
                });
                component.setOption(newOptions);
            } else {
                newOptions.xAxis.data.push('Não há dados');
                newOptions.series[0].data.push(100);
                newOptions.color = '#57737a';
                component.setOption(newOptions);
            }
        });
    }

    public async loadTarefasFinalizadas({ component }: IEventParam<IterableComponentRender>) {
        component.datasource.loading = true;

        const filter = FiltroUtil.montarFiltros(component.datasource.filter);
        const response = await HttpCRUDTranslator.get('/tarefasFinalizadas', filter);
        component.datasource.updateData(response.data.data);

        component.datasource.loading = false;
        FiltroUtil.reloadResources(DashboardResources);
    }

    public async loadTarefasAtrasadas({ component }: IEventParam<IterableComponentRender>) {
        component.datasource.loading = true;

        const filter = FiltroUtil.montarFiltros(component.datasource.filter);
        const response = await HttpCRUDTranslator.get('/tarefasAtrasadas', filter);
        component.datasource.updateData(response.data.data);

        component.datasource.loading = false;
    }

    public async loadTarefasConcluidas({ component }: IEventParam<IterableComponentRender>) {
        component.datasource.loading = true;

        const filter = FiltroUtil.montarFiltros(component.datasource.filter);
        const response = await HttpCRUDTranslator.get('/tarefasConcluidas', filter);
        component.datasource.updateData(response.data.data);

        component.datasource.loading = false;
    }

    public async loadUltimasTarefasRealizadas({ component }: IEventParam<IterableComponentRender>) {
        component.datasource.loading = true;

        const filter = FiltroUtil.montarFiltros(component.datasource.filter);
        const response = await HttpCRUDTranslator.get('/ultimasTarefasRealizadas', filter);
        component.datasource.updateData(response.data.data);

        component.datasource.loading = false;
    }
}
