import {
    Button,
    Image,
    IterableComponentRender, LoadingService, Modal, ModalService, Select, SelectMultiple, Text,
    Textarea,
    TextInput,
} from '@zeedhi/common';
import {
    Config, IDictionary, IEventParam, Metadata,
} from '@zeedhi/core';
import { HttpCRUDTranslator } from '@/http-crud-translator/http-crud-translator';
import { Utilities } from '@/util/Utilities';
import { IterableCarousel } from '@zeedhi/teknisa-components-common';
import { RichText } from '@zeedhi/zd-richtext';
import { FiltroUtil } from '../filtroUtil';

export class Fm00001OrdensServicoController {
    public modalDetalhesOrdem!: Modal;

    public visibilidadePaginaInicial: Boolean = true;

    public visibilidadePaginaExec: Boolean = false;

    public visibilidadePaginaRecusarOrdem: Boolean = false;

    public visibilidadePaginaCriarOrdem: Boolean = false;

    public currentCard: any;

    public modalExibirImagem!: Modal;

    public nr: any;

    public visibleCardsModeloPop: boolean | undefined = false;

    public modalModeloPop!: Modal;

    public modalOpcoesAberturaLink!: Modal;

    public modalFullscreen!: Modal;

    public visibilidadeBtnIniciar: Boolean = true;

    public visibilidadeBtnFinalizar: Boolean = false;

    public disableButton: Boolean = false;

    public visibilidadeBtnRecusar: Boolean = true;

    public visibilidadeBtnPausar: Boolean = false;

    public modalFiltro!: Modal;

    public tipoOrdenacao: String = '';

    private dadosModeloPop: any;

    public visibleCardsEpis: boolean | undefined = false;

    public async criarModais() {
        this.modalModeloPop = await ModalService.createFromJson('fm_00000_000_ordens_servico/modal/fm_0000_006_modelo_pop', true);
        this.modalOpcoesAberturaLink = await ModalService.createFromJson('fm_00000_000_ordens_servico/modal/fm_0000_005_opcoes_abertura', true);
        this.modalFiltro = await ModalService.createFromJson('fm_00000_000_ordens_servico/modal/fm_0000_007_filtro', true);
    }

    public async destruirModais() {
        this.modalModeloPop?.destroy();
        this.modalOpcoesAberturaLink?.destroy();
    }

    public async fecharModalFiltros() {
        this.modalFiltro.hide();
    }

    public fecharModalExibirImagens() {
        this.modalFullscreen.hide();
        this.modalFullscreen.destroy();
    }

    public async abrirModalFullscreen() {
        this.modalFullscreen = await ModalService.createFromJson('fm_00000_000_ordens_servico/modal/fm_0000_001_imagem_tela_cheia', true);
        this.modalFullscreen.show();
    }

    public abrirModalFiltrar() {
        this.modalFiltro.show();
    }

    public fecharModalFullscreen() {
        this.modalFullscreen.hide();
        this.modalFullscreen?.destroy();
    }

    public onMountedOrdenacao({ component }: IEventParam<IterableComponentRender>) {
        const filter = [
            { name: 'NMTELA', operation: '=', value: 'Ordem de Servico' },
        ];
        HttpCRUDTranslator.get('/getOrdenacao', filter).then((response: IDictionary) => {
            if (response.data.data.length !== 0 && typeof response.data.data.length !== 'undefined') {
                component.datasource.clearOrder();
                component.datasource.addOrder(response.data.data[0].NMCAMPO, response.data.data[0].IDSENTIDO);
                this.exibirTipoOrdenacao(response.data.data[0].NMCAMPO);
                Utilities.showSuccessMessage();
            }
            this.loadOrdensServico({ component });
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        });
    }

    public async loadOrdensServico({ component }: IEventParam<IterableComponentRender>) {
        component.datasource.loading = true;

        const filter = FiltroUtil.montarFiltros(component.datasource.filter);
        const response = await HttpCRUDTranslator.get('/getOrdensServico', filter);
        component.datasource.updateData(response.data.data);
        component.datasource.clearFilter();

        component.datasource.loading = false;
        // FiltroUtil.reloadResources(OrdensResources);
    }

    public getDadosCardClicado(nrOrdemServ: any) {
        const { data } = Metadata.getInstance<IterableComponentRender>('iterable-ordem-servico').datasource;
        const dados: any = data?.find((row: any) => row.NRORDEMSERV === Number(nrOrdemServ));
        return dados;
    }

    public async abrirModalDetalhes() {
        this.modalDetalhesOrdem.show();
    }

    public fecharModalDetalhes() {
        this.modalDetalhesOrdem.hide();
    }

    public async abrirModalModeloPop() {
        await this.modalModeloPop.show();
        this.montarInformacoesModeloPop();
    }

    public fecharModalModeloPop() {
        this.modalModeloPop.hide();
    }

    public abrirModalOpcoesAbertura() {
        this.modalOpcoesAberturaLink.show();
    }

    public fecharModalOpcoesAbertura() {
        this.modalOpcoesAberturaLink.hide();
    }

    public retornarPagina() {
        const iterable = Metadata.getInstance<IterableComponentRender>('iterable-ordem-servico');
        this.visibilidadePaginaInicial = true;
        this.visibilidadePaginaExec = !this.visibilidadePaginaInicial;
        this.visibilidadePaginaCriarOrdem = !this.visibilidadePaginaInicial;
        this.loadOrdensServico({ component: iterable });
    }

    public cancelarRecusaOrdem() {
        const textarea = Metadata.getInstance<Textarea>('textarea-descrisao-recusa');
        this.visibilidadePaginaRecusarOrdem = false;
        this.visibilidadePaginaInicial = false;
        this.visibilidadePaginaExec = true;
        this.visibilidadePaginaCriarOrdem = false;
        textarea.value = '';
    }

    public stylizeStatus({ component }: IEventParam<Text>) {
        if (component.text === 'Pendente') {
            component.text = "<span  style='background-color:#dee2e6; color: #ff6700; font-size: 10px;'>P E N D E N T E</span>";
        }
        if (component.text === 'Finalizada') {
            component.text = "<span  style='background-color: #dee2e6; color: green; font-size: 10px;'>F I N A L I Z A D A</span>";
        }
        if (component.text === 'Em Andamento') {
            component.text = "<span  style='background-color: #dee2e6; color: #023e8a; font-size: 11px;'>EM ANDAMENTO</span>";
        }
        if (component.text === 'Recusada') {
            component.text = "<span  style='background-color: #dee2e6; color: red; font-size: 10px;'>R E C U S A D A</span>";
        }
        if (component.text === 'Pausada') {
            component.text = "<span  style='background-color: #dee2e6; color: orange; font-size: 10px;'>P A U S A D A</span>";
        }
    }

    public async detalhesOrdem({ component }: IEventParam<Button>) {
        const dados = this.getDadosCardClicado(component.name.split('_')[1]);

        await this.abrirModalDetalhes();
        LoadingService.show();

        Metadata.getInstance<Text>('titulo-dados-ordem').text = dados.NMORDEMSERV;
        Metadata.getInstance<Text>('info-equipamento').text = dados.NMEQUIPAMENTO;
        Metadata.getInstance<Text>('info-categoria').text = dados.NMCATEGORIAMANUT;
        Metadata.getInstance<Text>('info-tipo-manutencao').text = dados.NMTIPOMANUTENCAO;
        Metadata.getInstance<Text>('info-prioridade').text = dados.NMPRIORIDADEMANUT;
        Metadata.getInstance<Text>('info-escopo').text = dados.NMESCOPOMANUT;
        Metadata.getInstance<Text>('info-situacao').text = dados.NMIDSITUACAO;

        LoadingService.hide();
    }

    public async carregarFotosOrdemServ() {
        const component = Metadata.getInstance<IterableCarousel>('carousel-imagens-anexo-ordem-servico');
        const filter = [
            { name: 'NRORDEMSERV', operation: '=', value: this.currentCard.NRORDEMSERV },
        ];
        await HttpCRUDTranslator.get('/getImagensOrdemServ', filter).then((response: any) => {
            if (response.data.data.length === 0) {
                component.isVisible = false;
                Metadata.getInstance<Text>('sem-resultado-imagens-ordem').isVisible = true;
            } else {
                component.iterable.datasource.updateData(response.data.data);
            }
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        });
    }

    public async clickCard({ component }: IEventParam<IterableComponentRender>) {
        LoadingService.show();
        this.currentCard = await this.getDadosCardClicado(component.name.split('_')[1]);
        const inputNumeroOsPai = document.querySelector('[name="input-numero-os"].col');

        const metadataMap = {
            textTitulo: Metadata.getInstance<Text>('titulo-ordem'),
            inpSolicitante: Metadata.getInstance<TextInput>('input-solicitante'),
            inpDataSolicitacao: Metadata.getInstance<TextInput>('input-dt-solicitacao'),
            inpNumeroSolicitacao: Metadata.getInstance<TextInput>('input-numero-solicitação'),
            inpNumeroOS: Metadata.getInstance<TextInput>('input-numero-os'),
            inpEquipamento: Metadata.getInstance<TextInput>('input-equipamento'),
            inpCategoria: Metadata.getInstance<TextInput>('input-categoria'),
            inpTipo: Metadata.getInstance<TextInput>('input-tipo-manut'),
            inpPrioridade: Metadata.getInstance<TextInput>('input-prioridade'),
            inpStatus: Metadata.getInstance<TextInput>('input-status'),
            inpTecnico: Metadata.getInstance<SelectMultiple>('input-externo'),
            inpOperador: Metadata.getInstance<SelectMultiple>('input-responsavel'),
            inpEquipe: Metadata.getInstance<Select>('input-equipe'),
            inpDetalhes: Metadata.getInstance<Textarea>('textarea-detalhes'),
            inpLocal: Metadata.getInstance<TextInput>('input-local'),
            inpCliente: Metadata.getInstance<TextInput>('input-cliente'),
        };

        metadataMap.inpNumeroOS.value = this.currentCard.NRORDEMSERV;
        metadataMap.inpEquipamento.value = this.currentCard.NMEQUIPAMENTO;
        metadataMap.inpCategoria.value = this.currentCard.NMCATEGORIAMANUT;
        metadataMap.inpTipo.value = this.currentCard.NMTIPOMANUTENCAO;
        metadataMap.inpPrioridade.value = this.currentCard.NMPRIORIDADEMANUT;
        metadataMap.inpStatus.value = this.currentCard.NMIDSITUACAO;
        metadataMap.inpDetalhes.value = this.currentCard.DSORDEMSERV;
        metadataMap.textTitulo.text = `Ordem de Serviço Nº ${this.currentCard.NRORDEMSERV}`;
        metadataMap.inpLocal.value = this.currentCard.NMLOCALCLIENTE;
        metadataMap.inpCliente.value = this.currentCard.NMFANTCLIE;
        metadataMap.inpTecnico.isVisible = false;
        metadataMap.inpOperador.isVisible = false;
        metadataMap.inpEquipe.isVisible = false;
        metadataMap.inpDataSolicitacao.isVisible = false;
        metadataMap.inpSolicitante.isVisible = false;
        metadataMap.inpNumeroSolicitacao.isVisible = false;
        inputNumeroOsPai!.className = 'col col-auto';

        if (this.currentCard.OPERADORRESPONSAVEL) {
            metadataMap.inpOperador.updateItems(this.currentCard.OPERADORRESPONSAVEL.split(','));
            metadataMap.inpOperador.setValue(this.currentCard.OPERADORRESPONSAVEL.split(','));
            metadataMap.inpOperador.isVisible = true;
        }

        if (this.currentCard.PROFISSIONALEXTERNO) {
            metadataMap.inpTecnico.updateItems(this.currentCard.PROFISSIONALEXTERNO.split(','));
            metadataMap.inpTecnico.setValue(this.currentCard.PROFISSIONALEXTERNO.split(','));
            metadataMap.inpTecnico.isVisible = true;
        }

        if (this.currentCard.NMEQUIPE) {
            metadataMap.inpEquipe.updateItems([this.currentCard.NMEQUIPE]);
            metadataMap.inpEquipe.setValue(this.currentCard.NMEQUIPE);
            metadataMap.inpEquipe.isVisible = true;
        }

        if (this.currentCard.NMOPERADOR) {
            metadataMap.inpSolicitante.value = this.currentCard.NMOPERADOR;
            metadataMap.inpSolicitante.isVisible = true;
        }

        if (this.currentCard.DTSOLICITACAOSERV) {
            metadataMap.inpDataSolicitacao.value = this.currentCard.DTSOLICITACAOSERV;
            metadataMap.inpDataSolicitacao.isVisible = true;
        }

        if (this.currentCard.NRSOLICITACAOSERV) {
            metadataMap.inpNumeroSolicitacao.value = this.currentCard.NRSOLICITACAOSERV;
            metadataMap.inpNumeroSolicitacao.isVisible = true;
            inputNumeroOsPai!.className = 'col col-6';
        }

        await Promise.all([
            this.carregarFotosOrdemServ(),
            this.loadModeloPop(),
            this.montarBotaoOrdem(),
            this.carregarEpis(),
        ]).catch((error) => {
            Utilities.showErrorMessage(error);
        }).then(() => {
            this.visibilidadePaginaInicial = false;
            this.visibilidadePaginaExec = true;
        }).finally(() => {
            LoadingService.hide();
        });
    }

    public abrirRecusarOrdem() {
        const textTitulo = Metadata.getInstance<Text>('titulo-ordem-recusar');
        textTitulo.text = `Ordem de Serviço Nº ${this.currentCard.NRORDEMSERV}`;
        this.visibilidadePaginaRecusarOrdem = true;
        this.visibilidadePaginaExec = !this.visibilidadePaginaRecusarOrdem;
        this.visibilidadePaginaInicial = !this.visibilidadePaginaRecusarOrdem;
        this.visibilidadePaginaCriarOrdem = !this.visibilidadePaginaRecusarOrdem;
    }

    public recusarOrdem() {
        LoadingService.show();
        const iterableOrdemServico = Metadata.getInstance<IterableComponentRender>('iterable-ordem-servico');
        const motivos = Metadata.getInstance<Textarea>('textarea-descricao-recusa');
        HttpCRUDTranslator.post('/recusarOS', {
            NRORDEMSERV: this.currentCard.NRORDEMSERV,
            MOTIVO: motivos.value,
        }).catch((error) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            this.loadOrdensServico({ component: iterableOrdemServico });
            this.visibilidadePaginaInicial = true;
            this.visibilidadePaginaExec = false;
            this.visibilidadePaginaRecusarOrdem = false;
            this.visibilidadePaginaCriarOrdem = false;
            LoadingService.hide();
            Utilities.showSuccessMessage();
        });
    }

    public finalizarOrdem() {
        LoadingService.show();
        const iterableOrdemServico = Metadata.getInstance<IterableComponentRender>('iterable-ordem-servico');
        HttpCRUDTranslator.post('/finalizarOS', {
            NRORDEMSERV: this.currentCard.NRORDEMSERV,
            NRSOLICITACAOSERV: this.currentCard.NRSOLICITACAOSERV,
        }).then(() => {
            Utilities.showSuccessMessage();
        }).catch((error) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            this.loadOrdensServico({ component: iterableOrdemServico });
            this.visibilidadePaginaInicial = true;
            this.visibilidadePaginaExec = false;
            this.visibilidadePaginaRecusarOrdem = false;
            LoadingService.hide();
        });
    }

    public iniciarOrdem() {
        LoadingService.show();
        this.disableButton = true;
        HttpCRUDTranslator.post('/iniciarOS', {
            NRORDEMSERV: this.currentCard.NRORDEMSERV,
        }).then(() => {
            this.currentCard.IDSITUACAO = '2';
            this.montarBotaoOrdem();
        }).catch((error) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
            this.montarBotaoOrdem();
        });
    }

    public pausarOrdem() {
        LoadingService.show();
        this.disableButton = true;
        HttpCRUDTranslator.post('/pausarOS', {
            NRORDEMSERV: this.currentCard.NRORDEMSERV,
        }).then(() => {
            this.currentCard.IDSITUACAO = '5';
            this.montarBotaoOrdem();
        }).catch((error) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            LoadingService.hide();
            this.montarBotaoOrdem();
        });
    }

    public async abrirImagem({ component }: IEventParam<Image>) {
        await this.abrirModalFullscreen();
        const componentFullscreen = Metadata.getInstance<Image>('imagem_fullscreen');
        componentFullscreen.src = component.src;
    }

    //  FUNÇOES MODELO POP

    public clickModeloPop() {
        const link = this.dadosModeloPop[0].LINKMODELOPOP;
        const descricao = this.dadosModeloPop[0].DESCMODELOPOP;
        const linkArquivo = this.dadosModeloPop[0].DSCAMINHOARQ;

        const hasDescricao = descricao && descricao !== '<p></p>'; // Verifica se existe descrição válida

        // Cenário 1: Apenas o link
        if (link && !linkArquivo && !hasDescricao) {
            this.abrirLink();
        }

        // Cenário 2: Apenas o linkArquivo
        if (linkArquivo && !link && !hasDescricao) {
            this.abrirPdf();
        }

        // Cenário 3: Apenas link e linkArquivo
        if (link && linkArquivo && !hasDescricao) {
            this.abrirModalOpcoesAbertura();
        }

        // Cenário 4: Qualquer um dos dois (link ou linkArquivo) + descrição ou observação
        if ((link || linkArquivo) && (hasDescricao)) {
            this.abrirModalModeloPop(); // Abre o modal
        }
    }

    public abrirPdf() {
        const codigo = this.dadosModeloPop[0].NRANEXOMODELOPOP;

        const downloadUri = `${Config.env.endPoint}downLoadAttachment?ID=${codigo}`;
        window.open(downloadUri);
    }

    public abrirOpcoesLink() {
        const link = this.dadosModeloPop[0].LINKMODELOPOP;
        const linkArquivo = this.dadosModeloPop[0].DSCAMINHOARQ;

        if (link !== null && linkArquivo !== null) {
            this.abrirModalOpcoesAbertura();
        } else if (link !== null) {
            this.abrirLink();
        } else {
            this.abrirPdf();
        }
    }

    public abrirLink() {
        const link = this.dadosModeloPop[0].LINKMODELOPOP;

        if (this.isValidLink(link)) {
            this.openValidLink(link); // Abre o link em nova guia
        }
    }

    /**
     * Valida se um link é válido.
     * @param url - O link a ser validado.
     * @returns Retorna `true` se o link for válido, caso contrário `false`.
     */
    public isValidLink(url: string): boolean {
        try {
            const parsedUrl = new URL(url);
            return parsedUrl.protocol === 'http:' || parsedUrl.protocol === 'https:';
        } catch (error) {
            Utilities.alertMessage('O link inserido é inválido, e, por esse motivo, não foi possível abrir o Modelo POP. Solicite ao supervisor que revise o modelo de procedimento anexado a esta ordem de serviço.', 'error');
            return false;
        }
    }

    /**
     * Abre um link válido em uma nova guia.
     * @param url - O link a ser aberto.
     */
    public openValidLink(url: string): void {
        if (this.isValidLink(url)) {
            window.open(url, '_blank');
        } else {
            Utilities.alertMessage('O link inserido é inválido, e, por esse motivo, não foi possível abrir o Modelo POP. Solicite ao supervisor que revise o modelo de procedimento anexado a esta ordem de serviço.', 'error');
        }
    }

    public montarInformacoesModeloPop() {
        const campoDescricao = Metadata.getInstance<RichText>('descricao-observacao');
        const descricao = this.dadosModeloPop[0].DESCMODELOPOP;
        const observacao = this.dadosModeloPop[0].DSOBSERVACAO;

        const hasDescricao = descricao && descricao !== '<p></p>'; // Verifica se existe descrição válida
        const hasObservacao = observacao !== null; // Verifica se existe observação válida

        campoDescricao.value = null;
        if (hasDescricao && hasObservacao) {
            campoDescricao.value = `${descricao} <br>Informações complementares: ${observacao}`;
        } else if (hasDescricao) {
            campoDescricao.value = descricao;
        } else {
            campoDescricao.value = `Informações complementares: ${observacao}`;
        }
    }

    public async loadModeloPop() {
        const filter = [
            { name: 'NRORDEMSERV', operation: '=', value: this.currentCard.NRORDEMSERV },
        ];

        await HttpCRUDTranslator.get('/getModeloPopOrdemServ', filter).catch((error) => {
            Utilities.showErrorMessage(error);
        }).then((response) => {
            this.dadosModeloPop = response.data.data;
            this.visibleCardsModeloPop = (this.dadosModeloPop && this.dadosModeloPop.length > 0);
        });
    }

    public montarBotaoOrdem() {
        // Pendente
        if (this.currentCard.IDSITUACAO === '1') {
            this.visibilidadeBtnIniciar = true;
            this.visibilidadeBtnFinalizar = false;

            this.visibilidadeBtnRecusar = true;
            this.visibilidadeBtnPausar = false;

            this.disableButton = false;
        }

        // Pausada
        if (this.currentCard.IDSITUACAO === '5') {
            this.visibilidadeBtnIniciar = true;
            this.visibilidadeBtnFinalizar = false;

            this.visibilidadeBtnPausar = false;
            this.visibilidadeBtnRecusar = true;

            this.disableButton = false;
        }

        // Em Andamento
        if (this.currentCard.IDSITUACAO === '2') {
            this.visibilidadeBtnIniciar = false;
            this.visibilidadeBtnFinalizar = true;

            this.visibilidadeBtnPausar = true;
            this.visibilidadeBtnRecusar = false;

            this.disableButton = false;
        }

        // Finalizada ou Recusada
        if (this.currentCard.IDSITUACAO === '3' || this.currentCard.IDSITUACAO === '4') {
            this.disableButton = true;
        }
    }

    public async carregarEpis() {
        const iterableEpis = Metadata.getInstance<IterableComponentRender>('iterable-cards-epis');
        const filter = [
            { name: 'NRORDEMSERV', operation: '=', value: this.currentCard.NRORDEMSERV },
        ];
        await HttpCRUDTranslator.get('/getEpisOS', filter).then((response) => {
            iterableEpis.datasource.updateData(response.data.data);
            this.visibleCardsEpis = (iterableEpis.datasource?.data && iterableEpis.datasource?.data.length > 0);
        }).catch((error) => {
            Utilities.showErrorMessage(error);
        });
    }

    public filtrarCards() {
        const iterable = Metadata.getInstance<IterableComponentRender>('iterable-ordem-servico');

        const filtros = {
            NRPRIORIDADEMANUT: Metadata.getInstance<SelectMultiple>('filtro-prioridade').value,
            NRTIPOMANUTENCAO: Metadata.getInstance<SelectMultiple>('filtro-tipo-manut').value,
            NREQUIPAMENTO: Metadata.getInstance<SelectMultiple>('filtro-equipamento').value,
            IDSITUACAO: Metadata.getInstance<SelectMultiple>('filtro-situacao').value,
            NRCATEGORIAMANUT: Metadata.getInstance<SelectMultiple>('filtro-categoria-manut').value,
            IDATIVO: Metadata.getInstance<Select>('filtro-idativo').value,
            NREQUIPE: Metadata.getInstance<SelectMultiple>('filtro-equipe').value,
            CDOPERADOR: Metadata.getInstance<SelectMultiple>('filtro-operador').value,
            NRPROFISSIONALEXTERNO: Metadata.getInstance<SelectMultiple>('filtro-profissional-externo').value,
        };

        const filtrosFiltrados = FiltroUtil.filtrarFiltros(filtros);

        iterable.datasource.setFilter(filtrosFiltrados);
        this.loadOrdensServico({ component: iterable });

        this.fecharModalFiltros();
    }

    public limparFiltros() {
        const iterable = Metadata.getInstance<IterableComponentRender>('iterable-ordem-servico');

        Metadata.getInstance<SelectMultiple>('filtro-prioridade').value = null;
        Metadata.getInstance<SelectMultiple>('filtro-tipo-manut').value = null;
        Metadata.getInstance<SelectMultiple>('filtro-equipamento').value = null;
        Metadata.getInstance<SelectMultiple>('filtro-situacao').value = null;
        Metadata.getInstance<SelectMultiple>('filtro-categoria-manut').value = null;
        Metadata.getInstance<Select>('filtro-idativo').value = null;
        Metadata.getInstance<SelectMultiple>('filtro-equipe').value = null;
        Metadata.getInstance<SelectMultiple>('filtro-operador').value = null;
        Metadata.getInstance<SelectMultiple>('filtro-profissional-externo').value = null;

        iterable.datasource.clearFilter();
        this.loadOrdensServico({ component: iterable });
        this.fecharModalFiltros();
    }

    public ordenarCards({ component }: IEventParam<Button>) {
        const iterable = Metadata.getInstance<IterableComponentRender>('iterable-ordem-servico');

        iterable.datasource.clearOrder();
        if (component.name === 'botao-ordenar-ordens') {
            iterable.datasource.addOrder('NMORDEMSERV', 'ASC');
        } else if (component.name === 'botao-ordenar-data') {
            iterable.datasource.addOrder('DTSOLICITACAOSERV', 'DESC');
        } else {
            iterable.datasource.clearOrder();
        }

        const { order } = iterable.datasource;
        if (order) {
            const campo = (order.length === 0) ? '' : order[0].split('.')[0];
            this.exibirTipoOrdenacao(campo);
        }
        iterable.datasource.setPage(1);
    }

    public exibirTipoOrdenacao(name: any) {
        const visibilidadeText = Metadata.getInstance<Text>('titulo-ordenacao');

        switch (name) {
            case 'NMORDEMSERV':
                this.tipoOrdenacao = 'Ordenado por nome da Ordem';
                visibilidadeText.isVisible = true;
                break;
            case 'DTSOLICITACAOSERV':
                this.tipoOrdenacao = 'Ordenado por Data';
                visibilidadeText.isVisible = true;
                break;
            default:
                this.tipoOrdenacao = '';
                visibilidadeText.isVisible = false;
                break;
        }
    }

    public abrirCriarOrdemServico() {
        this.visibilidadePaginaCriarOrdem = true;
        this.visibilidadePaginaRecusarOrdem = false;
        this.visibilidadePaginaInicial = false;
        this.visibilidadePaginaExec = false;
    }

    public fecharCriarOrdemServico() {
        this.visibilidadePaginaCriarOrdem = false;
        this.visibilidadePaginaRecusarOrdem = false;
        this.visibilidadePaginaInicial = true;
        this.visibilidadePaginaExec = false;
    }
}
