import { HttpCRUDTranslator } from '@/http-crud-translator/http-crud-translator';
import { SolicitacaoServResources } from '@/server/config/Resources';
import {
    AlertService, FileInput, Form, LoadingService, Select,
} from '@zeedhi/common';
import { IDictionary, Metadata } from '@zeedhi/core';
import { Utilities } from '@/util/Utilities';
import { FiltroUtil } from './filtroUtil';
import FileUpload from './FileUpload';

export class FmtSolicitacaoServicoController {
    public responsavel!: any;

    public dataSolicitacao: string = new Date(Date.now()).toISOString();

    public preencherCategories({ component }: { component: Select }) {
        HttpCRUDTranslator.get('/findCategoriaManutencao').then((response) => {
            component.datasource.updateData(response.data.data);
        }).catch((err) => {
            AlertService.show({
                name: 'error-alert',
                timeout: 5000,
                color: 'error',
                text: err,
            });
        });
        FiltroUtil.reloadResources(SolicitacaoServResources);
    }

    public preencherEquipamentos({ component }: { component: Select }) {
        HttpCRUDTranslator.get('/getEquipamento').then((response) => {
            component.datasource.updateData(response.data.data);
        }).catch((err) => {
            AlertService.show({
                name: 'error-alert',
                timeout: 5000,
                color: 'error',
                text: err,
            });
        });
    }

    public preencherResponsavel() {
        this.responsavel = localStorage.getItem('LOGIN_USER_NAME');
    }

    public preencherImpactos({ component }: { component: Select }) {
        HttpCRUDTranslator.get('/getImpactoManutencao').then((response) => {
            component.datasource.updateData(response.data.data);
        }).catch((err) => {
            AlertService.show({
                name: 'error-alert',
                timeout: 5000,
                color: 'error',
                text: err,
            });
        });
    }

    public async saveSolicitacaoServico() {
        const form = Metadata.getInstance<Form>('formSolicitacao');
        const imgs = await this.updloadPictures('fileInput');

        if (Utilities.validateForm(form)) {
            LoadingService.show();
            HttpCRUDTranslator.post('/saveSolicitacaoServico', {
                NMSOLICITACAOSERV: form.value.inputNomeSolicitacao,
                NRCATEGORIAMANUT: form.value.inputTipoServico,
                NREQUIPAMENTO: form.value.inputEquipamento,
                NRIMPACTOMANUT: form.value.inputImpacto,
                DSSOLICITACAOSERV: form.value.inputDescricao,
                CDOPERSOLICITANTE: form.value.selectResponsavel,
                FOTOS: imgs,
            }).then(() => {
                Utilities.showSuccessMessage();
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                LoadingService.hide();
            });
        }
    }

    public async updloadPictures(name: string) {
        const fileInputObject = Metadata.getInstance<FileInput>(name).value;
        let arquivos: any = [];
        if (fileInputObject && fileInputObject.length > 0) {
            const types: IDictionary = ['image/jpg', 'image/jpeg', 'image/png'];
            const buildFilesPromises = fileInputObject.map((file: any) => {
                if (types.includes(file.type)) {
                    return FileUpload.buildFileSendObject(file);
                }
                return [];
            });
            await Promise.all(buildFilesPromises).then((files: any) => {
                arquivos = files;
            });
          }
        return arquivos;
    }
}
